package be.iminds.ilabt.jfed.espec.model;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

public class BasicExperimentSpecificationBuilder implements ExperimentSpecificationBuilder<BasicExperimentSpecificationBuilder> {
    @Nullable private String versionBase;
    @Nullable private ExperimentSpecification.Flavor versionFlavor;
    @Nonnull private List<String> extensions;
    @Nonnull private List<RspecSpec> rspecs;
    @Nonnull private List<DirSpec> dirs;
    @Nonnull private List<UploadSpec> uploads;
    @Nonnull private List<ExecuteSpec> executes;
    @Nullable private AnsibleSpec ansible;

    public BasicExperimentSpecificationBuilder() {
        this.extensions = new ArrayList<>();
        this.rspecs = new ArrayList<>();
        this.dirs = new ArrayList<>();
        this.uploads = new ArrayList<>();
        this.executes = new ArrayList<>();
        this.ansible = null;
    }

    public BasicExperimentSpecificationBuilder(ExperimentSpecification orig) {
        this.versionBase = orig.getVersionBase();
        this.versionFlavor = orig.getVersionFlavor();
        this.extensions = new ArrayList<>(orig.getExtensions());
        this.rspecs = new ArrayList<>(orig.getRspecs());
        this.dirs = new ArrayList<>(orig.getDirs());
        this.uploads = new ArrayList<>(orig.getUploads());
        this.executes = new ArrayList<>(orig.getExecutes());
        this.ansible = orig.getAnsible();
    }

    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder setVersion(@Nonnull String versionFull) {
        int versionDashIndex = versionFull.indexOf("-");
        if (versionDashIndex == -1)
            throw new IllegalArgumentException("Experiment Specification version must be a String " +
                    "of the form <base>-<flavor>");

        this.versionBase = versionFull.substring(0, versionDashIndex);
        String versionFlavorString = versionFull.substring(versionDashIndex+1);
        ExperimentSpecification.Flavor versionFlavor;
        String versionFlavorSearchString = versionFlavorString.trim().toUpperCase();
        try {
            this.versionFlavor = ExperimentSpecification.Flavor.valueOf(versionFlavorSearchString);
        } catch (IllegalArgumentException e) {
            throw new IllegalArgumentException("Experiment Specification version must be a String of the " +
                    "form <base>-<flavor>  but flavor \""+versionFlavorString+"\" " +
                    "(internally \""+versionFlavorSearchString+"\") is not known)");
        }
        return this;
    }

    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder setVersionBase(@Nonnull String versionBase) {
        this.versionBase = versionBase;
        return this;
    }

    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder setVersionFlavor(@Nonnull ExperimentSpecification.Flavor versionFlavor) {
        this.versionFlavor = versionFlavor;
        return this;
    }

    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder setExtensions(@Nonnull List<String> extensions) {
        this.extensions = extensions;
        return this;
    }
    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder setRspecs(@Nonnull List<RspecSpec> rspecs) {
        this.rspecs.clear();
        this.rspecs.addAll(rspecs);
        return this;
    }
    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder addRspecs(@Nonnull List<RspecSpec> rspecs) {
        this.rspecs.addAll(rspecs);
        return this;
    }
    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder setRspec(@Nonnull RspecSpec rspecs) {
        this.rspecs.clear();
        this.rspecs.add(rspecs);
        return this;
    }
    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder addRspec(@Nonnull RspecSpec rspecs) {
        this.rspecs.add(rspecs);
        return this;
    }


    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder setDirs(@Nonnull List<DirSpec> dirs) {
        this.dirs.clear();
        this.dirs.addAll(dirs);
        return this;
    }
    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder addDir(@Nonnull DirSpec dir) {
        this.dirs.add(dir);
        return this;
    }
    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder addDirs(@Nonnull List<DirSpec> dirs) {
        this.dirs.addAll(dirs);
        return this;
    }


    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder setUploads(@Nonnull List<UploadSpec> uploads) {
        this.uploads.clear();
        this.uploads.addAll(uploads);
        return this;
    }
    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder addUpload(@Nonnull UploadSpec upload) {
        this.uploads.add(upload);
        return this;
    }
    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder addUploads(@Nonnull List<UploadSpec> uploads) {
        this.uploads.addAll(uploads);
        return this;
    }


    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder setExecutes(@Nonnull List<ExecuteSpec> executes) {
        this.executes.clear();
        this.executes.addAll(executes);
        return this;
    }
    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder addExecute(@Nonnull ExecuteSpec execute) {
        this.executes.add(execute);
        return this;
    }
    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder addExecutes(@Nonnull List<ExecuteSpec> executes) {
        this.executes.addAll(executes);
        return this;
    }

    @Nonnull
    @Override
    public BasicExperimentSpecificationBuilder setAnsible(@Nullable AnsibleSpec ansible) {
        this.ansible = ansible;
        return this;
    }


    @Nonnull
    @Override
    public ExperimentSpecification create() {
        if (versionBase == null)
            throw new IllegalStateException("You must set the ExperimentSpecification version (base and flavor)");
        if (versionFlavor == null)
            throw new IllegalStateException("You must set the ExperimentSpecification version (base and flavor)");
        if (rspecs.isEmpty())
            throw new IllegalStateException("You must add at least one Rspec to the ExperimentSpecification");

        //TODO: parse config
        ESpecConfig config = new ESpecConfig(
                false, null,
                true, true,
                false);

        return new ExperimentSpecification(
                versionBase, versionFlavor, config, extensions,
                rspecs, dirs, uploads, executes,
                ansible);
    }


    @Nullable public String getVersionBase() { return versionBase; }
    @Nullable public ExperimentSpecification.Flavor getVersionFlavor() { return versionFlavor; }
    @Nonnull public List<String> getExtensions() { return extensions; }
    @Nonnull public List<RspecSpec> getRspecs() { return rspecs; }
    @Nonnull public List<DirSpec> getDirs() { return dirs; }
    @Nonnull public List<UploadSpec> getUploads() { return uploads; }
    @Nonnull public List<ExecuteSpec> getExecutes() { return executes; }
    @Override @Nullable public AnsibleSpec getAnsible() { return ansible; }
}
